"""Module for Quotex API constants."""

codes_asset = {
    "AUDCAD": 36,
    "AUDCAD_otc": 67,
    "AUDCHF": 37,
    "AUDCHF_otc": 68,
    "AUDJPY": 38,
    "AUDJPY_otc": 69,
    "AUDUSD": 40,
    "AUDUSD_otc": 71,
    "AXJAUD": 315,
    "AXP_otc": 291,
    "BA_otc": 292,
    "CADCHF": 41,
    "CADJPY_otc": 73,
    "CHIA50": 328,
    "DJIUSD": 317,
    "EURAUD": 44,
    "EURCAD": 45,
    "EURCHF": 46,
    "EURCHF_otc": 77,
    "EURGBP": 47,
    "EURGBP_otc": 78,
    "EURJPY": 48,
    "EURJPY_otc": 79,
    "EURSGD": 123,
    "EURUSD": 1,
    "EURUSD_otc": 66,
    "F40EUR": 318,
    "FB_otc": 187,
    "FTSGBP": 319,
    "GBPAUD": 51,
    "GBPAUD_otc": 81,
    "GBPCAD": 52,
    "GBPCHF": 53,
    "GBPJPY": 54,
    "GBPJPY_otc": 84,
    "GBPUSD": 56,
    "GEREUR": 316,
    "HSIHKD": 320,
    "IBXEUR": 321,
    "INTC_otc": 190,
    "IT4EUR": 326,
    "JNJ_otc": 296,
    "JPXJPY": 327,
    "MCD_otc": 175,
    "MSFT_otc": 176,
    "NDXUSD": 322,
    "NZDJPY_otc": 89,
    "NZDUSD": 60,
    "NZDUSD_otc": 90,
    "PFE_otc": 297,
    "SPXUSD": 323,
    "STXEUR": 325,
    "UKBrent_otc": 164,
    "USCrude_otc": 165,
    "USDCAD": 61,
    "USDCAD_otc": 91,
    "USDCHF": 62,
    "USDCHF_otc": 92,
    "USDJPY": 63,
    "USDJPY_otc": 93,
    "XAGUSD": 65,
    "XAGUSD_otc": 167,
    "XAUUSD": 2,
    "XAUUSD_otc": 169,
}
